import React, { useState } from "react";
import UheppHist from "./UheppHist.jsx";

const noVariationUsed = (uhepp) => {
  const main_names = uhepp.stacks.map(stack => stack.content.map(item => item.yields)).flat()
  const num_names = uhepp.ratio.map(item => item.numerator).flat()
  const den_names = uhepp.ratio.map(item => item.denominator).flat()
  const names = [main_names, den_names, num_names].flat()
    
  const varied_names = names.filter(name => (name && name.indexOf("/") != -1))
  return varied_names.length == 0
}

const variationList = (uhepp) => {
  return Array.from(new Set(Object.entries(uhepp.yields).map(([key, item]) => [
      ...Object.keys(item.var_up || {}),
      ...Object.keys(item.var_down || {})
  ]).flat())).sort()
}

const makeVariationStacks = (uhepp, variation, updown) => {
  return  uhepp.stacks.map(stack => 
          ({
            type: "step", 
            content: [{
              "label": updown,
              "yield": stack.content.map(item => 
                item.yield.map(name => `${name}/${variation}/${updown}`)
              ).flat()
            }]
          })
   )
}

const UheppHistUI = ({width, height, uhepp}) => {
  const [uhepp_data, setData] = useState(uhepp)

  const handleRebin = (e) => {
    let values = Array.from(e.target.selectedOptions, option => parseFloat(option.value))
    setData(Object.assign({}, uhepp_data,
      {bins: Object.assign({}, uhepp_data.bins, {rebin: values})}
    ))
  }

  const handleUnderflow = (e) => {
    let value = e.target.checked
    setData(Object.assign({}, uhepp_data,
      {bins: Object.assign({}, uhepp_data.bins, {include_underflow: value})}
    ))
  }
  const handleOverflow = (e) => {
    let value = e.target.checked
    setData(Object.assign({}, uhepp_data,
      {bins: Object.assign({}, uhepp_data.bins, {include_overflow: value})}
    ))
  }
  const handleDensityWidth = (e) => {
    let value = parseFloat(e.target.value)
    setData(Object.assign({}, uhepp_data,
      {bins: Object.assign({}, uhepp_data.bins, {density_width: value})}
    ))
  }

  const handleEnvelop = (e) => {
    let variationName = e.target.value
    setData(Object.assign({}, uhepp_data,
      {stacks: [
        ...uhepp.stacks,
        ...makeVariationStacks(uhepp, variationName, "up"),
        ...makeVariationStacks(uhepp, variationName, "down"),
      ]}
    ))

  }

  const variations = variationList(uhepp)
  const isVariationReady = noVariationUsed(uhepp) && (variations.length > 0)

  const rebin = uhepp_data.bins.rebin || uhepp_data.bins.edges
  return <>
		<div>{ Object.entries(uhepp.metadata.tags).map(([key,  value]) => 
				value == null ?
				<span className="badge badge-pill badge-primary mx-1">{key}</span> :
				<span className="badge-pair mx-1">
					<span className="badge badge-pill badge-primary">{key}</span>
					<span className="badge badge-pill badge-secondary">{value}</span>
				</span>
				) }</div>
    <UheppHist width={width} height={height} uhepp={uhepp_data} />

		<ul className="nav nav-tabs" id="view-options" role="tablist">
			<li className="nav-item">
				<a className="nav-link active" id="info-tab" data-toggle="tab" href="#info" role="tab" aria-controls="info" aria-selected="true">Info</a>
			</li>
			<li className="nav-item">
				<a className="nav-link" id="binning-tab" data-toggle="tab" href="#binning" role="tab" aria-controls="binning" aria-selected="true">Binning</a>
			</li>
			{ <li className="nav-item">
				<a className="nav-link" id="variations-tab" data-toggle="tab" href="#variations" role="tab" aria-controls="variations" aria-selected="false">Variations</a>
			</li> }
			<li className="nav-item">
				<a className="nav-link" id="reset-tab" data-toggle="tab" href="#reset" role="tab" aria-controls="reset" aria-selected="false">Reset</a>
			</li>
		</ul>

		<div className="tab-content" id="view-options-content">
			<div className="tab-pane show active p-3" id="info" role="tabpanel" aria-labelledby="binning-tab">
				<dl>
					<dt>Author</dt>
					<dd>{ uhepp.metadata.author || <i>None</i>}</dd>
					<dt>Date</dt>
					<dd>{ uhepp.metadata.date ? (new Date(uhepp.metadata.date)).toString() : <i>None</i>}</dd>
					<dt>Producer</dt>
					<dd>{ uhepp.metadata.producer || <i>None</i>}</dd>
					<dt>Code revision</dt>
					<dd>{ uhepp.metadata.code_revision || <i>None</i>}</dd>
					<dt>Event selection</dt>
					<dd>{ uhepp.metadata.event_selection || <i>None</i>}</dd>
					<dt>Total integrated luminosity</dt>
					<dd>{ uhepp.metadata.lumi_ifb ?
								<span>{ uhepp.metadata.lumi_ifb} fb<sup>-1</sup></span> : <i>None</i>}</dd>
					<dt>Center-of-mass energy</dt>
					<dd>{ uhepp.metadata.Ecm_TeV ?
								uhepp.metadata.Ecm_TeV + " TeV" : <i>None</i>}</dd>
					<dt>Tags</dt>
					<dd>{ Object.entries(uhepp.metadata.tags).map(([key,  value]) => 
							value == null ?
							<span className="badge badge-pill badge-primary mx-1">{key}</span> :
							<span className="badge-pair mx-1">
								<span className="badge badge-pill badge-primary">{key}</span>
								<span className="badge badge-pill badge-secondary">{value}</span>
							</span>
							) }</dd>
				</dl>
			</div>
			<div className="tab-pane p-3" id="binning" role="tabpanel" aria-labelledby="binning-tab">
				<form>
					<div className="form-group">
						<label htmlFor="rebin">Bin edges</label>
						<select multiple size={10} className="form-control" id="rebin" onChange={(e) => handleRebin(e)}
                defaultValue={rebin}>
							{uhepp.bins.edges.map((v, i) => 
								<option value={v} key={i}>{v}</option>
							)}
						</select>
					</div>

					<div className="form-check">
						<input className="form-check-input" type="checkbox"
							checked={uhepp_data.bins.include_underflow == true} id="checkUnderflow"
							onChange={e => handleUnderflow(e)} />
						<label className="form-check-label" htmlFor="checkUnderflow">
							Include underflow events in first bin
						</label>
					</div>

					<div className="form-check">
						<input className="form-check-input" type="checkbox"
							checked={uhepp_data.bins.include_overflow == true} id="checkOverflow" 
							onChange={e => handleOverflow(e)} />
						<label className="form-check-label" htmlFor="checkOverflow">
							Include overflow events in last bin
						</label>
					</div>

					<div className="form-group">
						<label htmlFor="densityWidth">Normalize yields to</label>
						<div className="input-group">
							<input type="number" className="form-control" id="densityWidth"
								onChange={e => handleDensityWidth(e)}
								value={uhepp_data.bins.density_width || "0"} />
              { uhepp.variable.unit &&
                <div className="input-group-append">
                  <span className="input-group-text">{uhepp_data.variable.unit}</span>
                </div>
              }
						</div>
					</div>
				</form>
			</div>
			<div className="tab-pane p-3" id="variations" role="tabpanel" aria-labelledby="variations-tab">
        { !isVariationReady && 
          <p>To use the variation feature, you must add variation data to
          the <code>yields</code> object. Please note that the stacks must not use
          any varied yield in the default view.</p> }
        { isVariationReady && 
        <form>
					<div className="form-group">
						<label htmlFor="envelop">Envelop</label>
						<select className="form-control" id="envelop" onChange={(e) => handleEnvelop(e)}>
              { variations.map((name, i) => <option value={name} key={i}>{name}</option>) }
            </select>
          </div>
        </form>
        }
			</div>
			<div className="tab-pane p-3" id="reset" role="tabpanel" aria-labelledby="reset-tab">
        <p>Reset all view modifications. Pull the plot to make permanent
        changes.</p>
				<button className="btn btn-secondary m-1" type="button" onClick={() => setData(uhepp)}>
					Reset view
				</button>
			</div>
		</div>
  </>
}
export default UheppHistUI;
